<?php

/**
 * Plugin Abonnements
 * (c) 2012 Les Développements Durables
 * Licence GNU/GPL v3
 */

if (!defined('_ECRIRE_INC_VERSION')) { return;
}

include_spip('inc/actions');
include_spip('inc/editer');

/*
 * Déclaration des champs du formulaire
 */
function formulaires_editer_abonnement_saisies_dist($id_abonnement = 'new', $retour = '', $lier_trad = 0, $config_fonc = '', $row = [], $hidden = '') {
	$id_abonnement = intval($id_abonnement);

	// Si c'est une demande de création et qu'on trouve une offre, on ne doit donner que l'utilisateur et l'abonnement est défini caché
	if ($id_abonnement == 0 and $id_abonnements_offre = _request('id_abonnements_offre')) {
		$saisies = [
			[
				'saisie' => 'hidden',
				'options' => [
					'nom' => 'id_abonnements_offre',
					'defaut' => $id_abonnements_offre,
				],
			],
			[
				'saisie' => 'auteurs',
				'options' => [
					'nom' => 'id_auteur',
					'label' => _T('abonnement:champ_id_auteur_label'),
					'obligatoire' => 'oui',
					'cacher_option_intro' => 'oui',
				],
			],
		];
	}
	// Sinon si c'est une demande de création et qu'on trouve un auteur, on ne doit donner que les offres et l'utilisateur est défini caché
	elseif ($id_abonnement == 0 and $id_auteur = _request('id_auteur')) {
		$saisies = [
			[
				'saisie' => 'hidden',
				'options' => [
					'nom' => 'id_auteur',
					'defaut' => $id_auteur,
				],
			],
			[
				'saisie' => 'abonnements_offres',
				'options' => [
					'nom' => 'id_abonnements_offre',
					'label' => _T('abonnement:champ_id_abonnements_offre_label'),
					'obligatoire' => 'oui',
					'cacher_option_intro' => 'oui',
				],
			],
		];
	}
	// Sinon si c'est une demande de modification, on ne peut modifier que les dates
	elseif ($id_abonnement > 0) {
		$saisies = [
			[
				'saisie' => 'date',
				'options' => [
					'nom' => 'date_debut',
					'label' => _T('abonnement:champ_date_debut_label'),
					'horaire' => 'on',
					'heure_pas' => 5,
				],
				'verifier' => [
					'type' => 'date',
					'options' => [
						'normaliser' => 'datetime',
					],
				],
			],
			[
				'saisie' => 'date',
				'options' => [
					'nom' => 'date_echeance',
					'label' => _T('abonnement:champ_date_echeance_label'),
					'horaire' => 'on',
					'heure_pas' => 5,
				],
				'verifier' => [
					'type' => 'date',
					'options' => [
						'normaliser' => 'datetime',
					],
				],
			],
			[
				'saisie' => 'date',
				'options' => [
					'nom' => 'date_fin',
					'label' => _T('abonnement:champ_date_fin_label'),
					'horaire' => 'on',
					'heure_pas' => 5,
				],
				'verifier' => [
					'type' => 'date',
					'options' => [
						'normaliser' => 'datetime',
					],
				],
			],

			// afficher en readonly les infos d'echeance
			[
				'saisie' => 'input',
				'options' => [
					'nom' => 'echeance_duree',
					'label' => _T('abonnement:champ_echeance_duree_label'),
					'defaut' => 0,
					'readonly' => 'readonly',
					'conteneur_class' => 'disabled',
				],
				'verifier' => [
					'type' => 'entier',
					'options' => [
						'min' => 0,
					],
				],
			],
			[
				'saisie' => 'selection',
				'options' => [
					'nom' => 'echeance_periode',
					'label' => _T('abonnement:champ_echeance_periode_label'),
					'cacher_option_intro' => 'oui',
					'data' => [
						'mois' => _T('abonnementsoffre:champ_periode_choix_mois'),
						'jours' => _T('abonnementsoffre:champ_periode_choix_jours'),
						'heures' => _T('abonnementsoffre:champ_periode_choix_heures'),
					],
					'defaut' => 'mois',
					'readonly' => 'readonly',
					'conteneur_class' => 'disabled',
				],
			],
			[
				'saisie' => 'input',
				'options' => [
					'nom' => 'echeance_prix',
					'label' => _T('abonnement:champ_echeance_prix_label'),
					'defaut' => 0,
					'readonly' => 'readonly',
					'conteneur_class' => 'disabled',
				],
				'verifier' => [
					'type' => 'decimal',
					'options' => [
						'min' => 0,
					],
				],
			],
			[
				'saisie' => 'input',
				'options' => [
					'nom' => 'bank_uid',
					'label' => _T('abonnement:champ_bank_uid_label'),
					'defaut' => '',
					'readonly' => 'readonly',
					'conteneur_class' => 'disabled',
				],
			],
			[
				'saisie' => 'input',
				'options' => [
					'nom' => 'mode_paiement',
					'label' => _T('abonnement:champ_mode_paiement_label'),
					'defaut' => '',
					'readonly' => 'readonly',
					'conteneur_class' => 'disabled',
				],
			],
		];
	}

	return $saisies;
}

/**
 * Identifier le formulaire en faisant abstraction des parametres qui ne representent pas l'objet edite
 */
function formulaires_editer_abonnement_identifier_dist($id_abonnement = 'new', $retour = '', $lier_trad = 0, $config_fonc = '', $row = [], $hidden = '') {
	return serialize([intval($id_abonnement)]);
}

/**
 * Declarer les champs postes et y integrer les valeurs par defaut
 */
function formulaires_editer_abonnement_charger_dist($id_abonnement = 'new', $retour = '', $lier_trad = 0, $config_fonc = '', $row = [], $hidden = '') {
	$valeurs = formulaires_editer_objet_charger('abonnement', $id_abonnement, '', $lier_trad, $retour, $config_fonc, $row, $hidden);
	unset($valeurs['id_abonnement']);
	return $valeurs;
}

/**
 * Verifier les champs postes et signaler d'eventuelles erreurs
 */
function formulaires_editer_abonnement_verifier_dist($id_abonnement = 'new', $retour = '', $lier_trad = 0, $config_fonc = '', $row = [], $hidden = '') {
	$erreurs = [];//formulaires_editer_objet_verifier('abonnement',$id_abonnement);

	// Pour une création on vérifie l'offre
	if (
		!intval($id_abonnement)
		and (
			!$id_abonnements_offre = _request('id_abonnements_offre')
			or !sql_getfetsel('id_abonnements_offre', 'spip_abonnements_offres', 'id_abonnements_offre = ' . intval($id_abonnements_offre))
		)
	) {
		$erreurs['message_erreur'] = _T('abonnement:erreur_id_abonnements_offre');
	}

	return $erreurs;
}

/**
 * Traiter les champs postes
 */
function formulaires_editer_abonnement_traiter_dist($id_abonnement = 'new', $retour = '', $lier_trad = 0, $config_fonc = '', $row = [], $hidden = '') {
#	// Si c'est pour une modif on appelle l'allongement
#	if (intval($id_abonnement) > 0) {
#		// On récupère les champs qui ne font pas partie de la table
#		$duree = _request('duree');
#		$periode = _request('periode');
#		// On appelle l'action de modif d'échéance
#		$modifier_echeance = charger_fonction('modifier_echeance_abonnement', 'action/');
#		$modifier_echeance("$id_abonnement/$duree/$periode");
#	}
	// Normaliser les dates vides pour éviter une erreur SQL
	foreach (['date_debut', 'date_echeance', 'date_fin'] as $champ_date) {
		if (
			$date = _request($champ_date)
			and is_array($date)
			and !count(array_filter(array_values($date)))
		) {
			set_request($champ_date, '0000-00-00 00:00:00');
		}
	}

	$retours = formulaires_editer_objet_traiter('abonnement', $id_abonnement, '', $lier_trad, $retour, $config_fonc, $row, $hidden);

	return $retours;
}